@@ -1,5 +1,7 @@
+#!/usr/bin/env python3
 """Topology components for FAUCET Mininet unit tests."""
 
+from collections import namedtuple
 import os
 import socket
 import string
@@ -7,27 +9,53 @@ import shutil
 import subprocess
 import time
 
+import netifaces
+
 # pylint: disable=too-many-arguments
 
-from mininet.log import output
+from mininet.log import output, warn
 from mininet.topo import Topo
 from mininet.node import Controller
 from mininet.node import CPULimitedHost
 from mininet.node import OVSSwitch
+from mininet.link import TCIntf, Link
 
 from clib import mininet_test_util
 
-import netifaces
 
 # TODO: this should be configurable (e.g for randomization)
 SWITCH_START_PORT = 5
 
 
+class FaucetIntf(TCIntf):
+    """TCIntf that doesn't complain unnecessarily"""
+
+    def delete(self):
+        """Ignore interface deletion failure;
+           this is common after a veth pair has been deleted
+           on the other side."""
+        self.cmd('ip link del', self.name, '|| true')
+        self.node.delIntf(self)
+        self.link = None
+
+
+class FaucetLink(Link):
+    """Link using FaucetIntfs"""
+
+    # TODO: This can be simplified when we update Mininet in the test image
+    def __init__(self, node1, node2, port1=None, port2=None,
+                 intfName1=None, intfName2=None,
+                 addr1=None, addr2=None, **params):
+        Link.__init__(self, node1, node2, port1=port1, port2=port2,
+                      intfName1=intfName1, intfName2=intfName2,
+                      cls1=FaucetIntf, cls2=FaucetIntf,
+                      addr1=addr1, addr2=addr2,
+                      params1=params, params2=params)
+
+
 class FaucetHost(CPULimitedHost):
     """Base Mininet Host class, for Mininet-based tests."""
 
-    pass
-
 
 class FaucetSwitch(OVSSwitch):
     """Switch that will be used by all tests (netdev based OVS)."""
@@ -38,9 +66,43 @@ class FaucetSwitch(OVSSwitch):
     }
 
     def __init__(self, name, **params):
-        super(FaucetSwitch, self).__init__(
+        super().__init__(
             name=name, reconnectms=8000, **params)
 
+    @staticmethod
+    def _workaround(args):
+        """Workarounds/hacks for errors resulting from
+           cmd() calls within Mininet"""
+        # Workaround: ignore ethtool errors on tap interfaces
+        # This allows us to use tap tunnels as cables to switch ports,
+        # for example to test against OvS in a VM.
+        if (len(args) > 1 and args[0] == 'ethtool -K' and
+                getattr(args[1], 'name', '').startswith('tap')):
+            return True
+        return False
+
+    def cmd(self, *args, success=0, **kwargs):
+        """Commands typically must succeed for proper switch operation,
+           so we check the exit code of the last command in *args.
+           success: desired exit code (or None to skip check)"""
+        # pylint: disable=arguments-differ
+        cmd_output = super().cmd(*args, **kwargs)
+        exit_code = int(super().cmd('echo $?'))
+        if success is not None and exit_code != success:
+            msg = "%s exited with (%d):'%s'" % (args, exit_code, cmd_output)
+            if self._workaround(args):
+                warn('Ignoring:', msg, '\n')
+            else:
+                raise RuntimeError(msg)
+        return cmd_output
+
+    def attach(self, intf):
+        "Attach an interface and set its port"
+        super().attach(intf)
+        # This should be done in Mininet, but we do it for now
+        port = self.ports[intf]
+        self.cmd('ovs-vsctl set Interface', intf, 'ofport_request=%s' % port)
+
     def start(self, controllers):
         # Transcluded from Mininet source, since need to insert
         # controller parameters at switch creation time.
@@ -90,7 +152,7 @@ class NoControllerFaucetSwitch(FaucetSwitch):
     """A switch without any controllers (typically for remapping hardware to software."""
 
     def start(self, _controllers):
-        super(NoControllerFaucetSwitch, self).start(controllers=[])
+        super().start(controllers=[])
 
 
 class VLANHost(FaucetHost):
@@ -98,18 +160,21 @@ class VLANHost(FaucetHost):
 
     intf_root_name = None
 
-    def config(self, vlan=100, **params):
+    def config(self, vlans=[100], **params):  # pylint: disable=arguments-differ
         """Configure VLANHost according to (optional) parameters:
-           vlan: VLAN ID for default interface"""
-        super_config = super(VLANHost, self).config(**params)
+           vlans (list): List of VLAN IDs for default interface"""
+        super_config = super().config(**params)
         intf = self.defaultIntf()
-        vlan_intf_name = '%s.%d' % (intf, vlan)
-        for cmd in (
-                'ip -4 addr flush dev %s' % intf,
-                'ip -6 addr flush dev %s' % intf,
-                'vconfig add %s %d' % (intf, vlan),
-                'ip link set dev %s up' % vlan_intf_name,
-                'ip -4 addr add %s dev %s' % (params['ip'], vlan_intf_name)):
+        vlan_intf_name = '%s.%s' % (intf, '.'.join(str(v) for v in vlans))
+        cmds = [
+            'ip -4 addr flush dev %s' % intf,
+            'ip -6 addr flush dev %s' % intf,
+            'ip link set dev %s up' % vlan_intf_name,
+            'ip -4 addr add %s dev %s' % (params['ip'], vlan_intf_name)
+        ]
+        for v in vlans:
+            cmds.append('vconfig add %s %d' % (intf, v))
+        for cmd in cmds:
             self.cmd(cmd)
         self.intf_root_name = intf.name
         intf.name = vlan_intf_name
@@ -123,21 +188,28 @@ class FaucetSwitchTopo(Topo):
     CPUF = 0.5
     DELAY = '1ms'
 
+    def __init__(self, *args, **kwargs):
+        self.dpid_names = {}  # maps dpids to switch names
+        self.switch_dpids = {}  # maps switch names to dpids
+        self.switch_ports = {}  # maps switch names to port lists
+        self.dpid_port_host = {}  # maps switch hosts to ports
+        super().__init__(*args, **kwargs)
+
     @staticmethod
     def _get_sid_prefix(ports_served):
         """Return a unique switch/host prefix for a test."""
         # Linux tools require short interface names.
-        id_chars = string.ascii_letters + string.digits # pytype: disable=module-attr
+        id_chars = ''.join(sorted(string.ascii_letters + string.digits))  # pytype: disable=module-attr
         id_a = int(ports_served / len(id_chars))
         id_b = ports_served - (id_a * len(id_chars))
         return '%s%s' % (
             id_chars[id_a], id_chars[id_b])
 
-    def _add_tagged_host(self, sid_prefix, tagged_vid, host_n):
+    def _add_tagged_host(self, sid_prefix, tagged_vids, host_n):
         """Add a single tagged test host."""
         host_name = 't%s%1.1u' % (sid_prefix, host_n + 1)
         return self.addHost(
-            name=host_name, cls=VLANHost, vlan=tagged_vid, cpu=self.CPUF)
+            name=host_name, cls=VLANHost, vlans=tagged_vids, cpu=self.CPUF)
 
     def _add_untagged_host(self, sid_prefix, host_n, inNamespace=True): # pylint: disable=invalid-name
         """Add a single untagged test host."""
@@ -149,123 +221,100 @@ class FaucetSwitchTopo(Topo):
         host_name = 'e%s%1.1u' % (sid_prefix, host_n + 1)
         return self.addHost(name=host_name, cls=e_cls, host_n=host_n, tmpdir=tmpdir)
 
-    def _add_faucet_switch(self, sid_prefix, dpid, ovs_type, switch_cls):
+    def _add_faucet_switch(self, sid_prefix, dpid, hw_dpid, ovs_type):
         """Add a FAUCET switch."""
+        switch_cls = FaucetSwitch
         switch_name = 's%s' % sid_prefix
+        self.switch_dpids[switch_name] = dpid
+        self.dpid_names[dpid] = switch_name
+        if hw_dpid and hw_dpid == dpid:
+            remap_dpid = str(int(dpid) + 1)
+            output('bridging hardware switch DPID %s (%x) dataplane via OVS DPID %s (%x)\n' % (
+                dpid, int(dpid), remap_dpid, int(remap_dpid)))
+            dpid = remap_dpid
+            switch_cls = NoControllerFaucetSwitch
         return self.addSwitch(
             name=switch_name,
             cls=switch_cls,
             datapath=ovs_type,
             dpid=mininet_test_util.mininet_dpid(dpid))
 
-    def _add_links(self, switch, hosts, links_per_host, start_port):
-        port = start_port
-        for host in hosts:
-            for i in range(links_per_host):
-                # Order of switch/host is important, since host may be in a container.
-                self.addLink(switch, host, port1=port, delay=self.DELAY, use_htb=True)
-                port += 1
-        return port
+    # Hardware switch port virtualization through
+    # transparent OVS attachment bridge/patch panel
+    #
+    # Since FAUCET is talking to the hardware switch, it needs
+    # to use the hardware switch's OpenFlow ports, rather than
+    # the OpenFlow ports of the (transparent) OVS attachment bridge.
 
+    def hw_remap_port(self, dpid, port):
+        """Map OVS attachment bridge port number -> HW port number if necessary"""
+        if dpid != self.hw_dpid:
+            return port
+        assert self.hw_ports
+        return self.hw_ports[port - self.start_port]
+
+    peer_link = namedtuple('peer_link', 'port peer_dpid peer_port')
+
+    def hw_remap_peer_link(self, dpid, link):
+        """Remap HW port numbers -> OVS port numbers in link if necessary"""
+        port = self.hw_remap_port(dpid, link.port)
+        peer_port = self.hw_remap_port(link.peer_dpid, link.peer_port)
+        return self.peer_link(port, link.peer_dpid, peer_port)
+
+    def dpid_ports(self, dpid):
+        """Return port list for dpid, remapping if necessary"""
+        name = self.dpid_names[dpid]
+        ports = self.switch_ports[name]
+        return [self.hw_remap_port(dpid, port) for port in ports]
+
+    @staticmethod
+    def extend_port_order(port_order=None, max_length=16):
+        """Extend port_order to max_length if needed"""
+        if not port_order:
+            port_order = []
+        return port_order + list(range(len(port_order), max_length + 1))
+
+    def _add_links(self, switch, dpid, hosts, links_per_host):
+        self.switch_ports.setdefault(switch, [])
+        self.dpid_port_host.setdefault(int(dpid), {})
+        index = 0
+        for host in hosts:
+            for _ in range(links_per_host):
+                # Order of switch/host is important, since host may be in a container.
+                port = self.start_port + self.port_order[index]
+                self.addLink(switch, host, port1=port, delay=self.DELAY, use_htb=True)
+                # Keep track of switch ports
+                self.switch_ports.setdefault(switch, [])
+                self.switch_ports[switch].append(port)
+                self.dpid_port_host[int(dpid)][port] = host
+                index += 1
+        return index
+
+    # pylint: disable=too-many-locals,arguments-differ
     def build(self, ovs_type, ports_sock, test_name, dpids,
               n_tagged=0, tagged_vid=100, n_untagged=0, links_per_host=0,
-              n_extended=0, e_cls=None, tmpdir=None, hw_dpid=None,
-              host_namespace=None, start_port=SWITCH_START_PORT):
+              n_extended=0, e_cls=None, tmpdir=None, hw_dpid=None, switch_map=None,
+              host_namespace=None, start_port=SWITCH_START_PORT, port_order=None,
+              get_serialno=mininet_test_util.get_serialno):
         if not host_namespace:
             host_namespace = {}
-
+        self.hw_dpid = hw_dpid
+        self.hw_ports = sorted(switch_map) if switch_map else []
+        self.start_port = start_port
+        maxlength = n_tagged + n_untagged + n_extended
+        self.port_order = self.extend_port_order(port_order, maxlength)
         for dpid in dpids:
-            serialno = mininet_test_util.get_serialno(
-                ports_sock, test_name)
+            serialno = get_serialno(ports_sock, test_name)
             sid_prefix = self._get_sid_prefix(serialno)
-            for host_n in range(n_tagged):
-                self._add_tagged_host(sid_prefix, tagged_vid, host_n)
-            for host_n in range(n_untagged):
-                self._add_untagged_host(sid_prefix, host_n, host_namespace.get(host_n, True))
-            for host_n in range(n_extended):
-                self._add_extended_host(sid_prefix, host_n, e_cls, tmpdir)
-            switch_cls = FaucetSwitch
-            if hw_dpid and hw_dpid == dpid:
-                remap_dpid = str(int(dpid) + 1)
-                output('bridging hardware switch DPID %s (%x) dataplane via OVS DPID %s (%x)' % (
-                    dpid, int(dpid), remap_dpid, int(remap_dpid)))
-                dpid = remap_dpid
-                switch_cls = NoControllerFaucetSwitch
-            switch = self._add_faucet_switch(sid_prefix, dpid, ovs_type, switch_cls)
-            self._add_links(switch, self.hosts(), links_per_host, start_port)
-
-
-class FaucetStringOfDPSwitchTopo(FaucetSwitchTopo):
-    """String of datapaths each with hosts with a single FAUCET controller."""
-
-    switch_to_switch_links = 1
-
-    def build(self, ovs_type, ports_sock, test_name, dpids,
-              n_tagged=0, tagged_vid=100, n_untagged=0,
-              links_per_host=0, switch_to_switch_links=1,
-              hw_dpid=None, stack_ring=False, start_port=SWITCH_START_PORT):
-        """
-
-                               Hosts
-                               ||||
-                               ||||
-                 +----+       +----+       +----+
-              ---+1   |       |1234|       |   1+---
-        Hosts ---+2   |       |    |       |   2+--- Hosts
-              ---+3   |       |    |       |   3+---
-              ---+4  5+-------+5  6+-------+5  4+---
-                 +----+       +----+       +----+
-
-                 Faucet-1     Faucet-2     Faucet-3
-
-                   |            |            |
-                   |            |            |
-                   +-------- controller -----+
-
-        * s switches (above S = 3; for S > 3, switches are added to the chain)
-        * (n_tagged + n_untagged) hosts per switch
-        * (n_tagged + n_untagged + 1) links on switches 0 and s-1,
-          with final link being inter-switch
-        * (n_tagged + n_untagged + 2) links on switches 0 < n < s-1,
-          with final two links being inter-switch
-        """
-        def addLinks(src, dst, switch_ports): # pylint: disable=invalid-name
-            for _ in range(self.switch_to_switch_links):
-                self.addLink(src, dst, port1=switch_ports[src], port2=switch_ports[dst])
-                switch_ports[src] += 1
-                switch_ports[dst] += 1
-
-        first_switch = None
-        last_switch = None
-        self.switch_to_switch_links = switch_to_switch_links
-        switch_ports = {}
-        for dpid in dpids:
-            serialno = mininet_test_util.get_serialno(
-                ports_sock, test_name)
-            sid_prefix = self._get_sid_prefix(serialno)
-            hosts = []
-            for host_n in range(n_tagged):
-                hosts.append(self._add_tagged_host(sid_prefix, tagged_vid, host_n))
-            for host_n in range(n_untagged):
-                hosts.append(self._add_untagged_host(sid_prefix, host_n))
-            switch_cls = FaucetSwitch
-            if hw_dpid and hw_dpid == dpid:
-                remap_dpid = str(int(dpid) + 1)
-                output('bridging hardware switch DPID %s (%x) dataplane via OVS DPID %s (%x)' % (
-                    dpid, int(dpid), remap_dpid, int(remap_dpid)))
-                dpid = remap_dpid
-                switch_cls = NoControllerFaucetSwitch
-            switch = self._add_faucet_switch(sid_prefix, dpid, ovs_type, switch_cls)
-            if first_switch is None:
-                first_switch = switch
-            switch_ports[switch] = self._add_links(switch, hosts, links_per_host, start_port)
-            if last_switch is not None:
-                # Add a switch-to-switch link with the previous switch,
-                # if this isn't the first switch in the topology.
-                addLinks(last_switch, switch, switch_ports)
-            last_switch = switch
-        if stack_ring:
-            addLinks(first_switch, last_switch, switch_ports)
+            tagged = [self._add_tagged_host(sid_prefix, [tagged_vid], host_n)
+                      for host_n in range(n_tagged)]
+            untagged = [self._add_untagged_host(
+                sid_prefix, host_n, host_namespace.get(host_n, True))
+                        for host_n in range(n_untagged)]
+            extended = [self._add_extended_host(sid_prefix, host_n, e_cls, tmpdir)
+                        for host_n in range(n_extended)]
+            switch = self._add_faucet_switch(sid_prefix, dpid, hw_dpid, ovs_type)
+            self._add_links(switch, dpid, tagged + untagged + extended, links_per_host)
 
 
 class BaseFAUCET(Controller):
@@ -294,12 +343,13 @@ maximum_unreplied_echo_requests=5
 socket_timeout=15
 """
 
-    def __init__(self, name, tmpdir, controller_intf=None, controller_ipv6=False, cargs='', **kwargs):
+    def __init__(self, name, tmpdir, controller_intf=None, controller_ipv6=False,
+                 cargs='', **kwargs):
         name = '%s-%u' % (name, os.getpid())
         self.tmpdir = tmpdir
         self.controller_intf = controller_intf
         self.controller_ipv6 = controller_ipv6
-        super(BaseFAUCET, self).__init__(
+        super().__init__(
             name, cargs=self._add_cargs(cargs, name), **kwargs)
 
     def _add_cargs(self, cargs, name):
@@ -320,10 +370,10 @@ socket_timeout=15
         return ' '.join((
             self.BASE_CARGS, pid_file_arg, ryu_conf_arg, ofp_listen_host_arg, cargs))
 
-    def IP(self): # pylint: disable=invalid-name
+    def IP(self):  # pylint: disable=invalid-name,arguments-differ
         if self.controller_intf is not None:
             return self.controller_ip
-        return super(BaseFAUCET, self).IP()
+        return super().IP()
 
     def _start_tcpdump(self):
         """Start a tcpdump for OF port."""
@@ -435,7 +485,7 @@ socket_timeout=15
     def start(self):
         """Start tcpdump for OF port and then start controller."""
         self._start_tcpdump()
-        super(BaseFAUCET, self).start()
+        super().start()
 
     def _stop_cap(self):
         """Stop tcpdump for OF port and run tshark to decode it."""
@@ -455,7 +505,7 @@ socket_timeout=15
                     stderr=mininet_test_util.DEVNULL,
                     close_fds=True)
 
-    def stop(self):
+    def stop(self):  # pylint: disable=arguments-differ
         """Stop controller."""
         try:
             if self.CPROFILE:
@@ -465,7 +515,7 @@ socket_timeout=15
         except ProcessLookupError:
             pass
         self._stop_cap()
-        super(BaseFAUCET, self).stop()
+        super().stop()
         if os.path.exists(self.logname()):
             tmpdir_logname = os.path.join(
                 self.tmpdir, os.path.basename(self.logname()))
@@ -479,6 +529,7 @@ class FAUCET(BaseFAUCET):
 
     START_ARGS = ['--ryu-app=ryu.app.ofctl_rest']
 
+    # pylint: disable=too-many-locals
     def __init__(self, name, tmpdir, controller_intf, controller_ipv6, env,
                  ctl_privkey, ctl_cert, ca_certs,
                  ports_sock, prom_port, port, test_name, **kwargs):
@@ -489,7 +540,7 @@ class FAUCET(BaseFAUCET):
             '--ryu-wsapi-host=%s' % mininet_test_util.LOCALHOSTV6,
             '--ryu-wsapi-port=%u' % self.ofctl_port,
             self._tls_cargs(port, ctl_privkey, ctl_cert, ca_certs)))
-        super(FAUCET, self).__init__(
+        super().__init__(
             name,
             tmpdir,
             controller_intf,
@@ -503,7 +554,7 @@ class FAUCET(BaseFAUCET):
         return (
             self.listen_port(self.ofctl_port) and
             self.listen_port(self.prom_port) and
-            super(FAUCET, self).listening())
+            super().listening())
 
 
 class Gauge(BaseFAUCET):
@@ -512,7 +563,7 @@ class Gauge(BaseFAUCET):
     def __init__(self, name, tmpdir, controller_intf, controller_ipv6, env,
                  ctl_privkey, ctl_cert, ca_certs,
                  port, **kwargs):
-        super(Gauge, self).__init__(
+        super().__init__(
             name,
             tmpdir,
             controller_intf, controller_ipv6,
