@@ -2,7 +2,7 @@
 
 # Copyright (C) 2015 Brad Cowie, Christopher Lorier and Joe Stringer.
 # Copyright (C) 2015 Research and Education Advanced Network New Zealand Ltd.
-# Copyright (C) 2015--2018 The Contributors
+# Copyright (C) 2015--2019 The Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -67,10 +67,12 @@ The output action contains a dictionary with the following elements:
     defaults = {
         'rules': None,
         'exact_match': False,
+        'dot1x_assigned': False,
     }
     defaults_types = {
         'rules': list,
         'exact_match': bool,
+        'dot1x_assigned': bool,
     }
     rule_types = {
         'cookie': int,
@@ -105,9 +107,11 @@ The output action contains a dictionary with the following elements:
     def __init__(self, _id, dp_id, conf):
         self.rules = []
         self.exact_match = None
+        self.dot1x_assigned = None
         self.meter = False
         self.matches = {}
         self.set_fields = set()
+        self._ports_resolved = False
 
         #TODO: Would be possible to save the names instead of the DP and port objects
         # TUNNEL:
@@ -148,6 +152,10 @@ The output action contains a dictionary with the following elements:
             conf['rules'].append(normalized_rule)
         super(ACL, self).__init__(_id, dp_id, conf)
 
+    def finalize(self):
+        self._ports_resolved = True
+        super(ACL, self).finalize()
+
     def check_config(self):
         test_config_condition(
             not self.rules, 'no rules found for ACL %s' % self._id)
@@ -404,6 +412,8 @@ The output action contains a dictionary with the following elements:
         return result
 
     def resolve_ports(self, resolve_port_cb, resolve_tunnel_objects):
+        if self._ports_resolved:
+            return
         for rule_conf in self.rules:
             if 'actions' in rule_conf:
                 actions_conf = rule_conf['actions']
@@ -426,12 +436,19 @@ The output action contains a dictionary with the following elements:
                     else:
                         resolved_actions[action_name] = action_conf
                 rule_conf['actions'] = resolved_actions
+        self._ports_resolved = True
 
 
-# TODO: 802.1x steals the port ACL table.
+# NOTE: 802.1x steals the port ACL table.
 PORT_ACL_8021X = ACL(
     'port_acl_8021x', 0,
     {'rules': [{'eth_type': 1, 'eth_src': '01:02:03:04:05:06', 'actions': {
         'output': {'set_fields': [
             {'eth_src': '01:02:03:04:05:06'}, {'eth_dst': '01:02:03:04:05:06'}]}}}]})
 PORT_ACL_8021X.build({}, None, 1)
+
+MAB_ACL_8021X = ACL(
+    'mab_acl_8021x', 0,
+    {'rules': [{'eth_type': valve_of.ether.ETH_TYPE_IP, 'eth_src': '01:02:03:04:05:06',
+                'ip_proto': valve_of.inet.IPPROTO_UDP, 'udp_src': 68, 'udp_dst': 67, }]})
+MAB_ACL_8021X.build({}, None, 1)
