@@ -22,7 +22,7 @@ eventlet.monkey_patch()
 
 from ryu.lib import hub  # pylint: disable=wrong-import-position
 from chewie import chewie  # pylint: disable=wrong-import-position
-
+from faucet.valve_util import kill_on_exception
 
 def get_mac_str(valve_index, port_num):
     """Gets the mac address string for the valve/port combo
@@ -38,12 +38,15 @@ def get_mac_str(valve_index, port_num):
     return '00:00:00:%02x:%s' % (valve_index, two_byte_port_num_formatted)
 
 
-class FaucetDot1x:
+class FaucetDot1x:  # pylint: disable=too-many-instance-attributes
     """Wrapper for experimental Chewie 802.1x authenticator."""
 
-    def __init__(self, logger, metrics, send_flow_msgs):
+    exc_logname = None
+
+    def __init__(self, logger, exc_logname, metrics, send_flow_msgs):
         self.logger = logger
         self.metrics = metrics
+        self.exc_logname = exc_logname
         self.mac_to_port = {}  # {"00:00:00:00:00:02" : (valve_0, port_1)}
         self.dp_id_to_valve_index = {}
         self.thread = None
@@ -82,10 +85,10 @@ class FaucetDot1x:
         valve, port = self.mac_to_port[port_id]
         return (valve, port)
 
-    def _get_acls(self, dp):
+    def _get_acls(self, datapath):
         """Returns tuple of acl values"""
-        auth_acl = dp.acls.get(self._auth_acl_name)
-        noauth_acl = dp.acls.get(self._noauth_acl_name)
+        auth_acl = datapath.acls.get(self._auth_acl_name)
+        noauth_acl = datapath.acls.get(self._noauth_acl_name)
         return (auth_acl, noauth_acl)
 
     # Loggin Methods
@@ -100,24 +103,27 @@ class FaucetDot1x:
                                               'eth_src': mac_str,
                                               'status': status}})
 
-    def log_port_event(self, event_type, port_type, valve, port_num):
+    def log_port_event(self, event_type, port_type, valve, port_num):  # pylint: disable=no-self-use
         """Log a dot1x port event"""
         valve.dot1x_event({event_type: {'dp_id': valve.dp.dp_id,
                                         'port': port_num,
                                         'port_type': port_type}})
 
-    def auth_handler(self, address, port_id, vlan_name, filter_id):
+    @kill_on_exception(exc_logname)
+    def auth_handler(self, address, port_id, *args, **kwargs):  # pylint: disable=unused-argument
         """Callback for when a successful auth happens."""
         address_str = str(address)
         valve, dot1x_port = self._get_valve_and_port(port_id)
         port_num = dot1x_port.number
 
         self.log_auth_event(valve, port_num, address_str, 'success')
-        flowmods = self._get_login_flowmod(dot1x_port, valve, address_str, vlan_name)
-
+        flowmods = self._get_login_flowmod(dot1x_port, valve, address_str,
+                                           kwargs.get('vlan_name', None),
+                                           kwargs.get('filter_id', None))
         if flowmods:
             self._send_flow_msgs(valve, flowmods)
 
+    @kill_on_exception(exc_logname)
     def logoff_handler(self, address, port_id):
         """Callback for when an EAP logoff happens."""
         address_str = str(address)
@@ -131,6 +137,7 @@ class FaucetDot1x:
         if flowmods:
             self._send_flow_msgs(valve, flowmods)
 
+    @kill_on_exception(exc_logname)
     def failure_handler(self, address, port_id):
         """Callback for when a EAP failure happens."""
         address_str = str(address)
@@ -207,8 +214,32 @@ class FaucetDot1x:
 
         flowmods.extend(self._add_unauthenticated_flowmod(dot1x_port, valve))
 
+        if dot1x_port.dot1x_mab:
+            self.logger.info("Port % is using Mac Auth Bypass", dot1x_port.number)
+            flowmods.append(self.create_mab_flow(dp_id, dot1x_port, nfv_sw_port, valve))
+
         return flowmods
 
+    def create_mab_flow(self, dp_id, dot1x_port, nfv_sw_port, valve):
+        """Creates a flow that mirrors UDP packets from port 68 (DHCP) from
+        the supplicant to the nfv port
+
+        Args:
+            dp_id (int):
+            dot1x_port (Port):
+            nfv_sw_port (Port):
+            valve (Valve):
+
+        Returns:
+            list
+        """
+        acl_manager = valve.acl_manager
+        if dot1x_port.running():
+            valve_index = self.dp_id_to_valve_index[dp_id]
+            mac = get_mac_str(valve_index, dot1x_port.number)
+            return acl_manager.create_mab_flow(dot1x_port.number, nfv_sw_port.number, mac)
+        return []
+
     def create_flow_pair(self, dp_id, dot1x_port, nfv_sw_port, valve):
         """Creates the pair of flows that redirects the eapol packets to/from
         the supplicant and nfv port
@@ -222,10 +253,11 @@ class FaucetDot1x:
         Returns:
             list
         """
+        acl_manager = valve.acl_manager
         if dot1x_port.running():
             valve_index = self.dp_id_to_valve_index[dp_id]
             mac = get_mac_str(valve_index, dot1x_port.number)
-            return valve.create_dot1x_flow_pair(
+            return acl_manager.create_dot1x_flow_pair(
                 dot1x_port.number, nfv_sw_port.number, mac)
         return []
 
@@ -247,13 +279,15 @@ class FaucetDot1x:
         self._dot1x_speaker.port_down(mac)
 
         valve = self._valves[dp_id]
+        acl_manager = valve.acl_manager
         self.log_port_event("PORT_DOWN", 'supplicant', valve, port_num)
 
         flowmods = []
         flowmods.extend(self._del_authenticated_flowmod(dot1x_port, valve, mac))
         flowmods.extend(self._del_unauthenticated_flowmod(dot1x_port, valve))
         # NOTE: The flow_pair are not included in unauthed flowmod
-        flowmods.extend(valve.del_dot1x_flow_pair(dot1x_port.number, nfv_sw_port.number, mac))
+        flowmods.extend(acl_manager.del_mab_flow(dot1x_port.number, nfv_sw_port.number, mac))
+        flowmods.extend(acl_manager.del_dot1x_flow_pair(dot1x_port.number, nfv_sw_port.number, mac))
         return flowmods
 
     def reset(self, valves):
@@ -297,60 +331,78 @@ class FaucetDot1x:
             self._add_unauthenticated_flowmod(dot1x_port, valve))
         return flowmods
 
-    def _get_login_flowmod(self, dot1x_port, valve, mac_str, vlan_name):
+    def _get_login_flowmod(self, dot1x_port, valve,  # pylint: disable=too-many-arguments
+                           mac_str, vlan_name, acl_name):
         """Return flowmods required to login port"""
         flowmods = []
         flowmods.extend(
             self._del_unauthenticated_flowmod(dot1x_port, valve))
         flowmods.extend(
-            self._add_authenticated_flowmod(dot1x_port, valve, mac_str, vlan_name))
-
+            self._add_authenticated_flowmod(dot1x_port, valve, mac_str, vlan_name, acl_name))
         return flowmods
 
-    def _add_authenticated_flowmod(self, dot1x_port, valve, mac_str, vlan_name):
+    def _add_authenticated_flowmod(self, dot1x_port, valve,  # pylint: disable=too-many-arguments
+                                   mac_str, vlan_name, acl_name):
         """Return flowmods for successful authentication on port"""
         port_num = dot1x_port.number
         flowmods = []
+        acl_manager = valve.acl_manager
 
-        if dot1x_port.dot1x_acl:
+        acl = valve.dp.acls.get(acl_name, None)
+        if dot1x_port.dot1x_dyn_acl and acl:
+            self.logger.info("DOT1X_DYN_ACL: Adding ACL '{0}' for port '{1}'".format(
+                acl_name, port_num))
+            self.logger.debug("DOT1X_DYN_ACL: ACL contents: '{0}'".format(str(acl.__dict__)))
+            flowmods.extend(acl_manager.add_port_acl(acl, port_num, mac_str))
+        elif dot1x_port.dot1x_acl:
             auth_acl, _ = self._get_acls(valve.dp)
-            flowmods.extend(valve.add_port_acl(auth_acl, port_num, mac_str))
+            self.logger.info("DOT1X_PRE_ACL: Adding ACL '{0}' for port '{1}'".format(
+                acl_name, port_num))
+            self.logger.debug("DOT1X_PRE_ACL: ACL contents: '{0}'".format(str(auth_acl.__dict__)))
+            flowmods.extend(acl_manager.add_port_acl(auth_acl, port_num, mac_str))
         else:
-            flowmods.extend(valve.add_authed_mac(port_num, mac_str))
+            flowmods.extend(acl_manager.add_authed_mac(port_num, mac_str))
 
         if vlan_name:
-            flowmods.extend(valve.add_dot1x_native_vlan(port_num, mac_str, vlan_name))
-
+            flowmods.extend(valve.add_dot1x_native_vlan(port_num, vlan_name))
         return flowmods
 
     def _del_authenticated_flowmod(self, dot1x_port, valve, mac_str):
         """Return flowmods for deleting authentication flows from a port"""
         flowmods = []
         port_num = dot1x_port.number
+        acl_manager = valve.acl_manager
+
         if dot1x_port.dot1x_acl:
             auth_acl, _ = self._get_acls(valve.dp)
-            flowmods.extend(valve.del_port_acl(auth_acl, port_num, mac_str))
+            flowmods.extend(acl_manager.del_port_acl(auth_acl, port_num, mac_str))
+        elif dot1x_port.dot1x_dyn_acl:
+            flowmods.extend(acl_manager.del_authed_mac(port_num, mac_str, strict=False))
         else:
-            flowmods.extend(valve.del_authed_mac(port_num, mac_str))
+            flowmods.extend(acl_manager.del_authed_mac(port_num, mac_str))
 
-        flowmods.extend(valve.del_dot1x_native_vlan(port_num, mac_str))
+        flowmods.extend(valve.del_dot1x_native_vlan(port_num))
 
         return flowmods
 
     def _add_unauthenticated_flowmod(self, dot1x_port, valve, mac_str=None):
         """Return flowmods default on a port"""
         flowmods = []
+        acl_manager = valve.acl_manager
+
         if dot1x_port.dot1x_acl:
             _, noauth_acl = self._get_acls(valve.dp)
-            flowmods.extend(valve.add_port_acl(noauth_acl, dot1x_port.number, mac_str))
+            flowmods.extend(acl_manager.add_port_acl(noauth_acl, dot1x_port.number, mac_str))
 
         return flowmods
 
     def _del_unauthenticated_flowmod(self, dot1x_port, valve, mac_str=None):
         """Return flowmods for deleting default / unauthenticated flows from a port"""
         flowmods = []
+        acl_manager = valve.acl_manager
+
         if dot1x_port.dot1x_acl:
             _, noauth_acl = self._get_acls(valve.dp)
-            flowmods.extend(valve.del_port_acl(noauth_acl, dot1x_port.number, mac_str))
+            flowmods.extend(acl_manager.del_port_acl(noauth_acl, dot1x_port.number, mac_str))
 
         return flowmods
