@@ -3,7 +3,7 @@
 # Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
 # Copyright (C) 2015 Brad Cowie, Christopher Lorier and Joe Stringer.
 # Copyright (C) 2015 Research and Education Advanced Network New Zealand Ltd.
-# Copyright (C) 2015--2018 The Contributors
+# Copyright (C) 2015--2019 The Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import functools
 import ipaddress
 import random
 
@@ -38,9 +39,16 @@ VLAN_GROUP_OFFSET = MAX_VID + 1
 ROUTE_GROUP_OFFSET = VLAN_GROUP_OFFSET * 2
 OFP_VERSIONS = [ofp.OFP_VERSION]
 OFP_IN_PORT = ofp.OFPP_IN_PORT
-MAX_PACKET_IN_BYTES = 128
+MAX_PACKET_IN_BYTES = 194  # largest packet is icmp6 echo req with 128 byte payload
 ECTP_ETH_TYPE = 0x9000
 
+# https://en.wikipedia.org/wiki/IEEE_P802.1p
+# Avoid use of PCP 1 which is BK priority (lowest)
+PCP_EXT_PORT_FLAG = 2
+PCP_NONEXT_PORT_FLAG = 0
+EXTERNAL_FORWARDING_FIELD = 'vlan_pcp'
+
+
 OFERROR_TYPE_CODE = {
     0: ('OFPET_HELLO_FAILED', {
         ofp.OFPHFC_INCOMPATIBLE: 'OFPHFC_INCOMPATIBLE',
@@ -245,6 +253,7 @@ def is_packetout(ofmsg):
     """
     return isinstance(ofmsg, parser.OFPPacketOut)
 
+
 def is_output(ofmsg):
     """Return True if flow message is an action output message.
 
@@ -255,6 +264,7 @@ def is_output(ofmsg):
     """
     return isinstance(ofmsg, parser.OFPActionOutput)
 
+
 def is_flowdel(ofmsg):
     """Return True if flow message is a FlowMod and a delete.
 
@@ -269,6 +279,7 @@ def is_flowdel(ofmsg):
         return True
     return False
 
+
 def is_groupdel(ofmsg):
     """Return True if OF message is a GroupMod and command is delete.
 
@@ -282,6 +293,7 @@ def is_groupdel(ofmsg):
         return True
     return False
 
+
 def is_meterdel(ofmsg):
     """Return True if OF message is a MeterMod and command is delete.
 
@@ -377,6 +389,7 @@ def goto_table(table):
     """
     return parser.OFPInstructionGotoTable(table.table_id)
 
+
 def metadata_goto_table(metadata, mask, table):
     """Return instructions to write metadata and goto table.
 
@@ -391,6 +404,7 @@ def metadata_goto_table(metadata, mask, table):
         parser.OFPInstructionGotoTable(table.table_id)
         ]
 
+
 def set_field(**kwds):
     """Return action to set any field.
 
@@ -424,6 +438,7 @@ def devid_present(vid):
     return vid ^ ofp.OFPVID_PRESENT
 
 
+@functools.lru_cache(maxsize=1024)
 def push_vlan_act(table, vlan_vid, eth_type=ether.ETH_TYPE_8021Q):
     """Return OpenFlow action list to push Ethernet 802.1Q header with VLAN VID.
 
@@ -447,6 +462,7 @@ def dec_ip_ttl():
     return parser.OFPActionDecNwTtl()
 
 
+@functools.lru_cache(maxsize=1024)
 def pop_vlan():
     """Return OpenFlow action to pop outermost Ethernet 802.1Q VLAN header.
 
@@ -456,6 +472,7 @@ def pop_vlan():
     return parser.OFPActionPopVlan()
 
 
+@functools.lru_cache(maxsize=1024)
 def output_port(port_num, max_len=0):
     """Return OpenFlow action to output to a port.
 
@@ -468,6 +485,17 @@ def output_port(port_num, max_len=0):
     return parser.OFPActionOutput(port_num, max_len=max_len)
 
 
+def ports_from_output_port_acts(output_port_acts):
+    """Return unique port numbers from OFPActionOutput actions.
+
+    Args:
+        list of ryu.ofproto.ofproto_v1_3_parser.OFPActionOutput: output to port actions.
+    Returns:
+        set of port number ints.
+    """
+    return {output_port_act.port for output_port_act in output_port_acts}
+
+
 def dedupe_output_port_acts(output_port_acts):
     """Deduplicate parser.OFPActionOutputs (because Ryu doesn't define __eq__).
 
@@ -476,10 +504,40 @@ def dedupe_output_port_acts(output_port_acts):
     Returns:
         list of ryu.ofproto.ofproto_v1_3_parser.OFPActionOutput: output to port actions.
     """
-    output_ports = {output_port_act.port for output_port_act in output_port_acts}
+    output_ports = ports_from_output_port_acts(output_port_acts)
     return [output_port(port) for port in sorted(output_ports)]
 
 
+@functools.lru_cache(maxsize=1024)
+def output_non_output_actions(flood_acts):
+    """Split output actions into deduped actions, output ports, and non-output port actions.
+
+    Args:
+        list of ryu.ofproto.ofproto_v1_3_parser.OFPActions: flood actions.
+    Returns:
+        set of deduped actions, output ports, and non-output actions.
+    """
+    output_ports = set()
+    all_nonoutput_actions = set()
+    deduped_acts = []
+    # avoid dedupe_ofmsgs() here, as it's expensive - most of the time we are comparing
+    # port numbers as integers which is much cheaper.
+    for act in flood_acts:
+        if is_output(act):
+            if act.port in output_ports:
+                continue
+            output_ports.add(act.port)
+        else:
+            str_act = str(act)
+            if str_act in all_nonoutput_actions:
+                continue
+            all_nonoutput_actions.add(str_act)
+        deduped_acts.append(act)
+    nonoutput_actions = all_nonoutput_actions - set([str(pop_vlan())])
+    return (deduped_acts, output_ports, nonoutput_actions)
+
+
+@functools.lru_cache()
 def output_in_port():
     """Return OpenFlow action to output out input port.
 
@@ -489,6 +547,7 @@ def output_in_port():
     return output_port(OFP_IN_PORT)
 
 
+@functools.lru_cache()
 def output_controller(max_len=MAX_PACKET_IN_BYTES):
     """Return OpenFlow action to packet in to the controller.
 
@@ -499,8 +558,9 @@ def output_controller(max_len=MAX_PACKET_IN_BYTES):
     """
     return output_port(ofp.OFPP_CONTROLLER, max_len)
 
+
 def packetouts(port_nums, data):
-    """Return OpenFlow action to mulltiply packet out to dataplane from controller.
+    """Return OpenFlow action to multiply packet out to dataplane from controller.
 
     Args:
         port_num (list): ints, ports to output to.
@@ -529,6 +589,7 @@ def packetout(port_num, data):
     return packetouts([port_num], data)
 
 
+@functools.lru_cache()
 def barrier():
     """Return OpenFlow barrier request.
 
@@ -554,6 +615,7 @@ def match(match_fields):
     return parser.OFPMatch(**match_fields)
 
 
+@functools.lru_cache()
 def valve_match_vid(value):
     return to_match_vid(value, ofp.OFPVID_PRESENT)
 
@@ -604,13 +666,9 @@ MATCH_FIELDS = {
 
 
 def match_from_dict(match_dict):
-    for old_match, new_match in OLD_MATCH_FIELDS.items():
-        if old_match in match_dict:
-            match_dict[new_match] = match_dict[old_match]
-            del match_dict[old_match]
-
     kwargs = {}
     for of_match, field in match_dict.items():
+        of_match = OLD_MATCH_FIELDS.get(of_match, of_match)
         test_config_condition(of_match not in MATCH_FIELDS, 'Unknown match field: %s' % of_match)
         try:
             encoded_field = MATCH_FIELDS[of_match](field)
@@ -627,10 +685,11 @@ def _match_ip_masked(ipa):
     return (str(ipa.ip), str(ipa.netmask))
 
 
+@functools.lru_cache(maxsize=1024)
 def build_match_dict(in_port=None, vlan=None, eth_type=None, eth_src=None,
                      eth_dst=None, eth_dst_mask=None, icmpv6_type=None,
                      nw_proto=None, nw_dst=None, metadata=None,
-                     metadata_mask=None, vlan_pcp=None):
+                     metadata_mask=None, vlan_pcp=None, udp_src=None, udp_dst=None):
     match_dict = {}
     if in_port is not None:
         match_dict['in_port'] = in_port
@@ -654,6 +713,10 @@ def build_match_dict(in_port=None, vlan=None, eth_type=None, eth_src=None,
             match_dict['eth_dst'] = eth_dst
     if nw_proto is not None:
         match_dict['ip_proto'] = nw_proto
+    if udp_dst is not None:
+        match_dict['udp_dst'] = udp_dst
+    if udp_src is not None:
+        match_dict['udp_src'] = udp_src
     if icmpv6_type is not None:
         match_dict['icmpv6_type'] = icmpv6_type
     if nw_dst is not None:
@@ -768,7 +831,7 @@ def meterdel(datapath=None, meter_id=ofp.OFPM_ALL):
         meter_id)
 
 
-def meteradd(meter_conf):
+def meteradd(meter_conf, command=ofp.OFPMC_ADD):
     """Add a meter based on YAML configuration."""
 
     class NoopDP:
@@ -789,7 +852,7 @@ def meteradd(meter_conf):
             msg.xid = 0
 
     noop_dp = NoopDP()
-    ofctl.mod_meter_entry(noop_dp, meter_conf, ofp.OFPMC_ADD)
+    ofctl.mod_meter_entry(noop_dp, meter_conf, command)
     noop_dp.msg.xid = None
     noop_dp.msg.datapath = None
     return noop_dp.msg
@@ -814,23 +877,65 @@ def controller_pps_meterdel(datapath=None):
         meter_id=ofp.OFPM_CONTROLLER)
 
 
-def is_delete(ofmsg):
-    return is_flowdel(ofmsg) or is_groupdel(ofmsg) or is_meterdel(ofmsg)
+def slowpath_pps_meteradd(datapath=None, pps=0):
+    """Add a PPS meter towards controller."""
+    return parser.OFPMeterMod(
+        datapath=datapath,
+        command=ofp.OFPMC_ADD,
+        flags=ofp.OFPMF_PKTPS,
+        meter_id=ofp.OFPM_SLOWPATH,
+        bands=[parser.OFPMeterBandDrop(rate=pps)])
 
 
-_MSG_KINDS = (
-    ('packetout', is_packetout),
-    ('delete', is_delete),
-    ('tfm', is_table_features_req),
-    ('groupadd', is_groupadd),
-    ('meteradd', is_meteradd),
-)
+def slowpath_pps_meterdel(datapath=None):
+    """Delete a PPS meter towards controller."""
+    return parser.OFPMeterMod(
+        datapath=datapath,
+        command=ofp.OFPMC_DELETE,
+        flags=ofp.OFPMF_PKTPS,
+        meter_id=ofp.OFPM_SLOWPATH)
+
+
+def is_global_flowdel(ofmsg):
+    """Is a delete of all flows in all tables."""
+    return is_flowdel(ofmsg) and ofmsg.table_id == ofp.OFPTT_ALL and not ofmsg.match.items()
+
+
+def is_global_groupdel(ofmsg):
+    """Is a delete of all groups."""
+    return is_groupdel(ofmsg) and ofmsg.group_id == ofp.OFPG_ALL
+
+
+def is_global_meterdel(ofmsg):
+    """Is a delete of all meters."""
+    return is_meterdel(ofmsg) and ofmsg.meter_id == ofp.OFPM_ALL
+
+
+# We can tell right away what kind of OF messages these are.
+_MSG_KINDS_TYPES = {
+    parser.OFPPacketOut: 'packetout',
+    parser.OFPTableFeaturesStatsRequest: 'tfm',
+}
+
+
+# We need to examine the OF message more closely to classify it.
+_MSG_KINDS = {
+    parser.OFPFlowMod: (('deleteglobal', is_global_flowdel), ('delete', is_flowdel)),
+    parser.OFPGroupMod: (('deleteglobal', is_global_groupdel), ('delete', is_groupdel), ('groupadd', is_groupadd)),
+    parser.OFPMeterMod: (('deleteglobal', is_global_meterdel), ('delete', is_meterdel), ('meteradd', is_meteradd)),
+}
 
 
 def _msg_kind(ofmsg):
-    for kind, kind_func in _MSG_KINDS:
-        if kind_func(ofmsg):
-            return kind
+    ofmsg_type = type(ofmsg)
+    ofmsg_kind = _MSG_KINDS_TYPES.get(ofmsg_type, None)
+    if ofmsg_kind:
+        return ofmsg_kind
+    kinds = _MSG_KINDS.get(ofmsg_type, None)
+    if kinds:
+        for kind, kind_func in kinds:
+            if kind_func(ofmsg):
+                return kind
     return 'other'
 
 
@@ -842,23 +947,24 @@ def _partition_ofmsgs(input_ofmsgs):
     return by_kind
 
 
-def dedupe_ofmsgs(input_ofmsgs):
+def dedupe_ofmsgs(input_ofmsgs, random_order):
     """Return deduplicated ofmsg list."""
     # Built in comparison doesn't work until serialized() called
-    deduped_input_ofmsgs = []
-    input_ofmsgs_hashes = set()
-    for ofmsg in input_ofmsgs:
-        # Can't use dict or json comparison as may be nested
-        ofmsg_str = str(ofmsg)
-        if ofmsg_str in input_ofmsgs_hashes:
-            continue
-        deduped_input_ofmsgs.append(ofmsg)
-        input_ofmsgs_hashes.add(ofmsg_str)
-    return deduped_input_ofmsgs
+    # Can't use dict or json comparison as may be nested
+    deduped_input_ofmsgs = {str(ofmsg): ofmsg for ofmsg in input_ofmsgs}
+    if random_order:
+        ofmsgs = list(deduped_input_ofmsgs.values())
+        random.shuffle(ofmsgs)
+        return ofmsgs
+    # If priority present, send highest priority first.
+    return sorted(
+        deduped_input_ofmsgs.values(),
+        key=lambda ofmsg: getattr(ofmsg, 'priority', 2**16+1), reverse=True)
 
 
 # kind, random_order, suggest_barrier
 _OFMSG_ORDER = (
+    ('deleteglobal', False, True),
     ('delete', False, True),
     ('tfm', False, True),
     ('groupadd', False, True),
@@ -875,19 +981,16 @@ def valve_flowreorder(input_ofmsgs, use_barriers=True):
     # parallel delete will perform better and platforms that
     # don't will have at most only one barrier to deal with.
     output_ofmsgs = []
-    by_kind = _partition_ofmsgs(dedupe_ofmsgs(input_ofmsgs))
+    by_kind = _partition_ofmsgs(input_ofmsgs)
+
+    # Suppress all other deletes if a global delete is present.
+    delete_global_ofmsgs = by_kind.get('deleteglobal', [])
+    if delete_global_ofmsgs:
+        by_kind['delete'] = []
+
     for kind, random_order, suggest_barrier in _OFMSG_ORDER:
-        ofmsgs = by_kind.get(kind, [])
+        ofmsgs = dedupe_ofmsgs(by_kind.get(kind, []), random_order)
         if ofmsgs:
-            if random_order:
-                random.shuffle(ofmsgs)
-            else:
-                with_priorities = [ofmsg for ofmsg in ofmsgs if hasattr(ofmsg, 'priority')]
-                # If priority present, send highest priority first.
-                if with_priorities:
-                    with_priorities.sort(key=lambda ofmsg: ofmsg.priority, reverse=True)
-                    without_priorities = [ofmsg for ofmsg in ofmsgs if not hasattr(ofmsg, 'priority')]
-                    ofmsgs = without_priorities + with_priorities
             output_ofmsgs.extend(ofmsgs)
             if use_barriers and suggest_barrier:
                 output_ofmsgs.append(barrier())
@@ -897,6 +1000,9 @@ def valve_flowreorder(input_ofmsgs, use_barriers=True):
 def flood_tagged_port_outputs(ports, in_port=None, exclude_ports=None):
     """Return list of actions necessary to flood to list of tagged ports."""
     flood_acts = []
+    in_port_mirror_output_ports = {}
+    if in_port is not None:
+        in_port_mirror_output_ports = ports_from_output_port_acts(in_port.mirror_actions())
     if ports:
         for port in ports:
             if in_port is not None and port == in_port:
@@ -906,7 +1012,11 @@ def flood_tagged_port_outputs(ports, in_port=None, exclude_ports=None):
             if exclude_ports and port in exclude_ports:
                 continue
             flood_acts.append(output_port(port.number))
-            flood_acts.extend(port.mirror_actions())
+            # Only mirror if different mirror actions to in_port (port already will be mirrored on input).
+            mirror_actions = port.mirror_actions()
+            mirror_output_ports = ports_from_output_port_acts(mirror_actions)
+            if in_port is None or in_port_mirror_output_ports != mirror_output_ports:
+                flood_acts.extend(mirror_actions)
     return dedupe_output_port_acts(flood_acts)
 
 
